/**********************************************************************
 *
 * phrasequeryfilter.cpp -- 
 * Copyright (C) 1999  DigiLib Systems Limited, New Zealand
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: phrasequeryfilter.cpp 499 1999-08-31 22:44:38Z rjmcnab $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.3  1999/08/31 22:44:38  rjmcnab
   Fixed small bug.

   Revision 1.2  1999/07/07 06:19:45  rjmcnab
   Added ability to combine two or more independant queries.

   Revision 1.1  1999/07/01 04:01:47  rjmcnab
   Initial revision.


 */


#include "phrasequeryfilter.h"
#include "queryinfo.h"

#include "phrasesearch.h"



// do aditional query processing
void phrasequeryfilterclass::post_process (const queryparamclass &queryparams,
					   queryresultsclass &queryresults) {
  bool results_shrunk = false;

  // post-process the results if needed
  if (queryresults.orgterms.size() > 1 && !queryresults.docs.docset.empty()) {
    // get the long version of the index
    text_t longindex;
    indexmap.to2from (queryparams.index, longindex);
      
    // process each of the matched documents
    docresultmap::iterator docs_here = queryresults.docs.docset.begin();
    docresultmap::iterator docs_end = queryresults.docs.docset.end();
    while (docs_here != docs_end) {
      if ((*docs_here).second.num_query_terms_matched == queryresults.orgterms.size() &&
	  OID_phrase_search (*mgsearchptr, *gdbmptr, queryparams.index,
			     queryparams.subcollection, queryparams.language, 
			     longindex, queryparams.collection, queryresults.orgterms,
			     (*docs_here).second.docnum)) {
	(*docs_here).second.num_phrase_match++;
	docs_here++;

      } else {
	docresultmap::iterator this_doc_here = docs_here;
	docs_here++;
	if (queryparams.match_mode == 1) {
	  // need to delete this element, there was no exact match
	  queryresults.docs.docset.erase(this_doc_here);
	  results_shrunk = true;
	}
      }
    }
  }

  if (results_shrunk) queryresults.docs_matched = queryresults.docs.docset.size();
}


void phrasequeryfilterclass::sort_doc_results (const FilterRequest_t &/*request*/,
					 docresultsclass &docs) {
  resultsorderer_t resultsorderer;
  resultsorderer.compare_phrase_match = true;
  resultsorderer.compare_terms_match = true;
  resultsorderer.compare_doc_weight = true;
  resultsorderer.docset = &(docs.docset);

  // first get a list of document numbers
  docs.docnum_order();

  sort (docs.docorder.begin(), docs.docorder.end(), resultsorderer);
}
