/**********************************************************************
 *
 * queryfilter.h -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: queryfilter.h 396 1999-07-16 00:17:06Z sjboddie $
 *
 *********************************************************************/


#ifndef QUERYFILTER_H
#define QUERYFILTER_H


#include "gsdlconf.h"
#include "text_t.h"
#include "comtypes.h"
#include "filter.h"
#include "infodbclass.h"
#include "maptools.h"
#include "mgsearch.h"
#include "queryinfo.h"


// resultsorderer_t is used to sort the query results
struct resultsorderer_t {
  bool compare_phrase_match;
  bool compare_terms_match;
  bool compare_doc_weight;
  docresultmap *docset;

  resultsorderer_t();
  void clear();
  bool operator()(const int &t1, const int &t2) const;
};



class queryfilterclass : public filterclass {
protected:

  stringmap indexmap;
  stringmap subcollectionmap;
  stringmap languagemap;

  text_t gdbm_filename;
  gdbmclass *gdbmptr;
  
  mgsearchclass *mgsearchptr;


  void get_phrase_terms (const text_t &querystring, 
			 const termfreqclassarray &orgterms,
			 vector<termfreqclassarray> &phrases);

  // do aditional query processing
  virtual void post_process (const queryparamclass &queryparams,
			     queryresultsclass &queryresults);

  // get the query parameters
  void parse_query_params (const FilterRequest_t &request,
			   vector<queryparamclass> &query_params,
			   int &startresults,
			   int &endresults,
			   ostream &logout);

  // do query that might involve multiple sub queries
  // mgsearchptr and gdbmptr are assumed to be valid
  void do_multi_query (const FilterRequest_t &request,
		       const vector<queryparamclass> &query_params,
		       queryresultsclass &multiresults,
		       comerror_t &err, ostream &logout);

  virtual void sort_doc_results (const FilterRequest_t &request,
				 docresultsclass &docs);

  
public:
  queryfilterclass ();
  virtual ~queryfilterclass ();

  // the gdbmptr remains the responsability of the calling code
  void set_gdbmptr (gdbmclass *thegdbmptr) {gdbmptr=thegdbmptr;}

  // the mgsearchptr remains the responsability of the calling code
  void set_mgsearchptr (mgsearchclass *themgsearchptr) {mgsearchptr=themgsearchptr;}

  void configure (const text_t &key, const text_tarray &cfgline);
  bool init (ostream &logout);
  text_t get_filter_name () {return "QueryFilter";}
  void filter (const FilterRequest_t &request,
	       FilterResponse_t &response,
	       comerror_t &err, ostream &logout);
};


#endif
