/**********************************************************************
 *
 * cgiwrapper.cpp -- output pages using the cgi protocol
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: cgiwrapper.cpp 508 1999-09-02 00:24:36Z rjmcnab $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.19  1999/09/02 00:24:36  rjmcnab
   fixed bug in getting POST arguments

   Revision 1.18  1999/08/20 01:02:07  sjboddie
   added some usage logging

   Revision 1.17  1999/07/15 06:03:15  rjmcnab
   Moved the adding of the actions to librarymain so that they can be
   overriden easily.

   Revision 1.16  1999/07/14 08:31:05  rjmcnab
   Fixed a small bug in the POST implementation.

   Revision 1.15  1999/07/13 23:32:17  rjmcnab
   Added authenaction and usersaction

   Revision 1.14  1999/07/11 01:03:37  rjmcnab
   Added ability to receive POST cgi form data.

   Revision 1.13  1999/06/24 05:12:18  sjboddie
   lots of small changes

   Revision 1.12  1999/04/30 01:59:40  sjboddie
   lots of stuff - getting documentaction working (documentaction replaces
   old browseaction)

   Revision 1.11  1999/03/25 03:12:01  sjboddie

   subjectbrowseaction was replaced with browseaction

   Revision 1.10  1999/03/05 03:53:54  sjboddie

   fixed some bugs

   Revision 1.9  1999/03/04 22:38:21  sjboddie

   Added subjectbrowseaction. - Doesn't do anything yet.

   Revision 1.8  1999/02/28 20:00:13  rjmcnab


   Fixed a few things.

   Revision 1.7  1999/02/21 22:33:53  rjmcnab

   Lots of stuff :-)

   Revision 1.6  1999/02/12 02:40:17  sjboddie

   Added page action

   Revision 1.5  1999/02/11 01:24:04  rjmcnab

   Fixed a few compiler warnings.

   Revision 1.4  1999/02/08 01:28:01  rjmcnab

   Got the receptionist producing something using the statusaction.

   Revision 1.3  1999/02/05 10:42:44  rjmcnab

   Continued working on receptionist

   Revision 1.2  1999/02/04 10:00:56  rjmcnab

   Developed the idea of an "action" and having them define the cgi arguments
   which they need and how those cgi arguments function.

   Revision 1.1  1999/02/04 01:16:17  rjmcnab

   Initial revision.

   Revision 1.5  1999/01/19 01:38:18  rjmcnab

   Made the source more portable.

   Revision 1.4  1999/01/12 01:51:04  rjmcnab

   Standard header.

 */


#include "gsdlconf.h"
#include "cgiwrapper.h"
#include "recptconfig.h"
#include <stdlib.h>


// get GSDL_GSDLHOME
#include "gsdlhome.h"


#if defined(GSDL_USE_OBJECTSPACE)
#  include <ospace/std/iostream>
#  include <ospace/std/fstream>
#elif defined(GSDL_USE_IOS_H)
#  include <iostream.h>
#  include <fstream.h>
#else
#  include <iostream>
#  include <fstream>
#endif

#ifdef USE_FASTCGI
#include "fcgiapp.h"
#endif


#ifdef USE_FASTCGI
// used to output the text from receptionist
class fcgistreambuf : public streambuf {
public:
  fcgistreambuf ();
  int sync ();
  int overflow (int ch);
  int underflow () {return EOF;}
  
  void fcgisbreset() {fcgx_stream = NULL; other_ostream = NULL;};
  void set_fcgx_stream(FCGX_Stream *newone) {fcgx_stream=newone;};
  void set_other_ostream(ostream *newone) {other_ostream=newone;};
  
private:
  FCGX_Stream *fcgx_stream;
  ostream *other_ostream;
};

fcgistreambuf::fcgistreambuf() {
  fcgisbreset();
  if (base() == ebuf()) allocate();
  setp (base(), ebuf());
};

int fcgistreambuf::sync () {
  if ((fcgx_stream != NULL) &&
      (FCGX_PutStr (pbase(), out_waiting(), fcgx_stream) < 0)) {
    fcgx_stream = NULL;
  }

  if (other_ostream != NULL) {
    char *thepbase=pbase();
    for (int i=0;i<out_waiting();i++) (*other_ostream).put(thepbase[i]);
  }
  
  setp (pbase(), epptr());
  
  return 0;
}

int fcgistreambuf::overflow (int ch) {
  if (sync () == EOF) return EOF;
  if (ch != EOF) sputc (ch);
  return 0;
}

#endif


static void page_errorsitecfg (const text_t &gsdlhome, const text_t &collection,
			       text_t &errorpage) {
  errorpage += "Content-type: text/html\n\n";

  errorpage += "<html>\n";
  errorpage += "<head>\n";
  errorpage += "<title>Error</title>\n";
  errorpage += "</head>\n";
  errorpage += "<body>\n";
  errorpage += "<h2>Oops!</h2>\n";
  errorpage += "The site.cfg configuration file could not be found. This file\n";
  errorpage += "should contain configuration information relating to this\n";
  errorpage += "site's setup. ";
  if (collection.empty()) {
    errorpage += "As this cgi script is not being run in collection specific mode,\n";
    errorpage += "the file should reside at ";
    errorpage += gsdlhome;
    errorpage += "/etc/site.cfg.\n";
  } else {
    errorpage += "As this cgi script is being run in collection specific mode,\n";
    errorpage += "the file can reside in ";
    errorpage += gsdlhome;
    errorpage += "/collect/";
    errorpage += collection;
    errorpage += "/etc/site.cfg or ";
    errorpage += gsdlhome;
    errorpage += "/etc/site.cfg.\n";
  }
  errorpage += "</body>\n";
  errorpage += "</html>\n";
}


static void page_errormaincfg (const text_t &gsdlhome, const text_t &collection,
			       text_t &errorpage) {
  errorpage += "Content-type: text/html\n\n";

  errorpage += "<html>\n";
  errorpage += "<head>\n";
  errorpage += "<title>Error</title>\n";
  errorpage += "</head>\n";
  errorpage += "<body>\n";
  errorpage += "<h2>Oops!</h2>\n";
  if (collection.empty()) {
    errorpage += "The main.cfg configuration file could not be found. This file\n";
    errorpage += "should contain configuration information relating to the\n";
    errorpage += "setup of the interface. As this cgi script is not being run\n";
    errorpage += "in collection specific mode the file should reside at\n";
    errorpage += gsdlhome;
    errorpage += "/etc/main.cfg.\n";
  } else {
    errorpage += "Neither the collect.cfg or main.cfg configuration files could\n";
    errorpage += "not be found. This file should contain configuration information\n";
    errorpage += "relating to the setup of the interface. As this cgi script is\n";
    errorpage += "being run in collection specific mode the file should reside\n";
    errorpage += "at either ";
    errorpage += gsdlhome;
    errorpage += "/collect/";
    errorpage += collection;
    errorpage += "/etc/collect.cfg, ";
    errorpage += gsdlhome;
    errorpage += "/etc/collect.cfg or ";
    errorpage += gsdlhome;
    errorpage += "/etc/main.cfg.\n";
  }
  errorpage += "</body>\n";
  errorpage += "</html>\n";
}


static void page_errorinit (const text_t &/*gsdlhome*/, text_t &errorpage) {
  errorpage += "Content-type: text/html\n\n";

  errorpage += "<html>\n";
  errorpage += "<head>\n";
  errorpage += "<title>Error</title>\n";
  errorpage += "</head>\n";
  errorpage += "<body>\n";
  errorpage += "<h2>Oops!</h2>\n";
  errorpage += "An error occurred during the initialisation of the Greenstone Digital\n";
  errorpage += "Library software. It is likely that the software has not been setup\n";
  errorpage += "correctly.\n";

  ifstream initin (GSDL_GSDLHOME "/etc/initout.txt");
  if (initin) {
    errorpage += "The initialisation error log, " GSDL_GSDLHOME "/etc/initout.txt, contains the\n";
    errorpage += "following information:\n\n";
    errorpage += "<pre>\n";

    char c;
    initin.get(c);
    while (!initin.eof ()) {
      errorpage.push_back(c);
      initin.get(c);
    }
    
    errorpage += "</pre>\n";

    initin.close();

  } else {
    errorpage += "Please consult " GSDL_GSDLHOME "/etc/initout.txt for more information.\n";
  }

  errorpage += "</body>\n";
  errorpage += "</html>\n";
}

static void page_errorparseargs (const text_t &/*gsdlhome*/, text_t &errorpage) {
  errorpage += "Content-type: text/html\n\n";

  errorpage += "<html>\n";
  errorpage += "<head>\n";
  errorpage += "<title>Error</title>\n";
  errorpage += "</head>\n";
  errorpage += "<body>\n";
  errorpage += "<h2>Oops!</h2>\n";
  errorpage += "An error occurred during the parsing of the cgi arguments.\n";

  ifstream errin (GSDL_GSDLHOME "/etc/errout.txt");
  if (errin) {
    errorpage += "The error log, " GSDL_GSDLHOME "/etc/errout.txt, contains the\n";
    errorpage += "following information:\n\n";
    errorpage += "<pre>\n";

    char c;
    errin.get(c);
    while (!errin.eof ()) {
      errorpage.push_back(c);
      errin.get(c);
    }
    errorpage += "</pre>\n";
    errin.close();

  } else {
    errorpage += "Please consult " GSDL_GSDLHOME "/etc/errout.txt for more information.\n";
  }

  errorpage += "</body>\n";
  errorpage += "</html>\n";
}

static void page_errorcgipage (const text_t &/*gsdlhome*/, text_t &errorpage) {
  errorpage += "Content-type: text/html\n\n";

  errorpage += "<html>\n";
  errorpage += "<head>\n";
  errorpage += "<title>Error</title>\n";
  errorpage += "</head>\n";
  errorpage += "<body>\n";
  errorpage += "<h2>Oops!</h2>\n";
  errorpage += "An error occurred during the construction of the cgi page.\n";

  ifstream errin (GSDL_GSDLHOME "/etc/errout.txt");
  if (errin) {
    errorpage += "The error log, " GSDL_GSDLHOME "/etc/errout.txt, contains the\n";
    errorpage += "following information:\n\n";
    errorpage += "<pre>\n";

    char c;
    errin.get(c);
    while (!errin.eof ()) {
      errorpage.push_back(c);
      errin.get(c);
    }
    errorpage += "</pre>\n";
    errin.close();

  } else {
    errorpage += "Please consult " GSDL_GSDLHOME "/etc/errout.txt for more information.\n";
  }

  errorpage += "</body>\n";
  errorpage += "</html>\n";
}


// cgiwrapper does everything necessary to output a page
// using the cgi protocol. If this is being run for a particular
// collection then "collection" should be set, otherwise it
// should equal "".
void cgiwrapper (receptionist &recpt, text_t collection) {
#ifdef USE_FASTCGI
  fcgistreambuf outbuf;
#endif

  // init stuff - we can't output error pages directly with
  // fastcgi so the pages are stored until we can output them
  text_t errorpage;
  outconvertclass text_t2ascii;

  // set defaults
  int maxrequests = 10000;
  recpt.configure ("gsdlhome", GSDL_GSDLHOME);
  recpt.configure ("collection", collection);
  recpt.configure ("httpimg", "/gsdl/images");
  char *script_name = getenv("SCRIPT_NAME");
  if (script_name != NULL) recpt.configure("gwcgi", script_name);
  else recpt.configure("gwcgi", "/cgi-bin/gw");

  // read in the configuration files.
  if (!site_cfg_read (recpt, GSDL_GSDLHOME, collection, maxrequests)) {
    // couldn't find the site configuration file
    page_errorsitecfg (GSDL_GSDLHOME, collection, errorpage);
  } else if (!main_cfg_read (recpt, GSDL_GSDLHOME, collection)) {
    // couldn't find the main configuration file
    page_errormaincfg (GSDL_GSDLHOME, collection, errorpage);
  }

  // initialise the library software
  if (errorpage.empty()) {
    ofstream initout (GSDL_GSDLHOME "/etc/initout.txt");
    if (!recpt.init(initout)) {
      // an error occurred during the initialisation
      initout.close();
      page_errorinit(GSDL_GSDLHOME, errorpage);
    }
    initout.close();
  }
  
  // find out whether this is being run as a cgi-script
  // or a fastcgi script
  int numrequests = 0;
#ifdef USE_FASTCGI
  int isfastcgi = !FCGX_IsCGI();
  FCGX_Stream *fcgiin, *fcgiout, *fcgierr;
  FCGX_ParamArray fcgienvp;
#else
  int isfastcgi = 0;
#endif

  // get the query string if it is not being run as a fastcgi
  // script
  text_t argstr = "";
  cgiargsclass args;
  char *aURIStr;
  if (!isfastcgi) {
    char *request_method_str = getenv("REQUEST_METHOD");
    char *content_length_str = getenv("CONTENT_LENGTH");
    if (request_method_str != NULL && strcmp(request_method_str, "POST") == 0 &&
	content_length_str != NULL)  {
      // POST form data
      int content_length = text_t(content_length_str).getint();
      if (content_length > 0) {
	char c;
	do {
	  cin.get(c);
	  if (cin.eof()) break;
	  argstr.push_back (c);
	  content_length--;
	} while (content_length > 0);
      }
      
    } else {
      aURIStr = getenv("QUERY_STRING");
      if ((request_method_str != NULL && strcmp(request_method_str, "GET") == 0)
	  || aURIStr != NULL) {
	// GET form data
	if (aURIStr != NULL) argstr = aURIStr;
      } else {
	// debugging from command line
	char cinURIStr[1024];
	cin.get(cinURIStr, 1024);
	argstr = cinURIStr;
      }
    }

    // cgi scripts only deal with one request
    maxrequests = 1;
  }

  // Page-request loop. If this is not being run as a fastcgi
  // process then only one request will be processed and then
  // the process will exit.
  while (numrequests < maxrequests) {
#ifdef USE_FASTCGI
    if (isfastcgi) {
      if (FCGX_Accept(&fcgiin, &fcgiout, &fcgierr, &fcgienvp) < 0) break;
      aURIStr = FCGX_GetParam("QUERY_STRING", fcgienvp);
      if (aURIStr != NULL) argstr = aURIStr;
      else argstr = "";
    }
#endif

    // get output streams ready
#ifdef USE_FASTCGI
    outbuf.fcgisbreset ();
    if (isfastcgi) outbuf.set_fcgx_stream (fcgiout);
    else outbuf.set_other_ostream (&cout);
    ostream pageout (&outbuf);
#else
#define pageout cout
#endif

    if (errorpage.empty()) {
      ofstream errout (GSDL_GSDLHOME "/etc/errout.txt");
      cerr = errout;
    
      // parse the cgi arguments and produce the resulting page if there
      // has been no errors so far
      if (!recpt.parse_cgi_args (argstr, args, errout)) {
	errout.close ();
	page_errorparseargs(GSDL_GSDLHOME, errorpage);
      } else {
	if (!recpt.produce_cgi_page (args, pageout, errout)) {
	  errout.close ();
	  page_errorcgipage(GSDL_GSDLHOME, errorpage);
	} else {
	  errout.close ();
	}
	recpt.log_cgi_args (args, errout);
      }
    }
    // there was an error, output the error page
    if (!errorpage.empty()) {
      pageout << text_t2ascii << errorpage;
      errorpage.clear();
      numrequests = maxrequests; // make this the last page
    }
    pageout << flush;
    
    // finish with the output streams
#ifdef USE_FASTCGI
    if (isfastcgi) FCGX_Finish();
#endif

    numrequests++;
  }

  return;
}

