/**********************************************************************
 *
 * tipaction.cpp -- define random tip macros
 * Copyright (C) 1999  DigiLib Systems Limited, New Zealand
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: tipaction.cpp 515 1999-09-02 00:29:27Z rjmcnab $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.3  1999/09/02 00:29:27  rjmcnab
   added cgi argument "st" - show tips

   Revision 1.2  1999/07/30 02:24:45  sjboddie
   added collectinfo argument to some functions

   Revision 1.1  1999/07/09 02:18:27  rjmcnab
   Initial revision.


 */


#include "tipaction.h"
#include <stdlib.h>


tipaction::tipaction () {
  // "st" show tips - "0"=no, "1"=yes
  cgiarginfo arg_ainfo;
  arg_ainfo.shortname = "st";
  arg_ainfo.longname = "show tips";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "1";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (NULL, arg_ainfo);
}

void tipaction::define_external_macros (const ColInfoResponse_t &/*collectinfo*/, displayclass &disp, 
					cgiargsclass &args, recptproto */*collectproto*/, 
					ostream &logout) {
  // define_external_macros sets the following macros:

  // _tip:thistip_  - a random tip (defined if st == "1")

  if (args["st"].getint()) {
    // get the maximum tip number (_tip:tipmax_)
    text_t tipmax_str;
    disp.expandstring ("tip", "_tipmax_", tipmax_str);
    if (tipmax_str.empty()) {
      logout << "Warning: _tip:tipmax_ not set. This macro is needed for displaying tips.\n";
      return;
    }
    
    int tipmax = tipmax_str.getint();
    if (tipmax < 1) {
      logout << "Warning: _tip:tipmax_ was less than 1. No tips were set.\n";
      return;
    }
    
    disp.setmacro("thistip", "tip", "_tip" + text_t((rand()%tipmax) + 1) + "_");
  }
}

